Package: lakhesis
Title: Consensus Seriation for Binary Data
Version: 1.1.2
Authors@R: 
    person(c("Stephen", "A."), "Collins-Elliott", , "sce@utk.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-5642-6903"))
Description: Determining consensus seriations for binary incidence matrices, using a two-step process of Procrustes-fit correspondence analysis for heuristic selection of partial seriations and iterative regression to establish a single consensus. Contains the Lakhesis Calculator, a graphical platform for identifying seriated sequences. Collins-Elliott (2026) "Lakhesis: Consensus Seriation via Iterative Regression of Partial Rankings for Binary Data" <doi:10.1080/02664763.2026.2672564>.
License: GPL (>= 3)
Imports: stats, Rcpp, RcppArmadillo, readr, ca, ggplot2, Rdpack, shiny,
        shinydashboard, bslib
RdMacros: Rdpack
Encoding: UTF-8
RoxygenNote: 7.3.2
LinkingTo: Rcpp, RcppArmadillo
Depends: R (>= 3.5.0)
LazyData: true
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-05-28 17:17:18 UTC; archaeologus
Author: Stephen A. Collins-Elliott [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5642-6903>)
Maintainer: Stephen A. Collins-Elliott <sce@utk.edu>
Repository: CRAN
Date/Publication: 2026-05-28 17:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-28 21:36:33 UTC; unix
Archs: lakhesis.so.dSYM
