# Generated by roxygen2: do not edit by hand

S3method(ca_procrustes,default)
S3method(ca_procrustes,incidence_matrix)
S3method(ca_procrustes,matrix)
S3method(ca_procrustes_ser,incidence_matrix)
S3method(ca_procrustes_ser,matrix)
S3method(conc_c,incidence_matrix)
S3method(conc_c,matrix)
S3method(conc_wrc,incidence_matrix)
S3method(conc_wrc,matrix)
S3method(cor_sq,incidence_matrix)
S3method(cor_sq,matrix)
S3method(element_eval,incidence_matrix)
S3method(element_eval,matrix)
S3method(im_long,incidence_matrix)
S3method(im_long,matrix)
S3method(im_merge,incidence_matrix)
S3method(im_merge,matrix)
S3method(im_ref,incidence_matrix)
S3method(im_ref,matrix)
S3method(lakhesize,default)
S3method(lakhesize,strands)
S3method(plot,incidence_matrix)
S3method(plot,lakhesis)
S3method(plot,procrustean)
S3method(plot,strand)
S3method(print,lakhesis)
S3method(print,procrustean)
S3method(print,strand)
S3method(print,strands)
S3method(spearman_sq,numeric)
S3method(strand_add,strand)
S3method(strand_create,incidence_matrix)
S3method(strand_create,matrix)
S3method(strand_extract,strands)
S3method(strand_suppress,default)
S3method(strand_suppress,strands)
S3method(strands_create,list)
S3method(summary,lakhesis)
S3method(summary,strand)
export(LC)
export(ca_procrustes)
export(ca_procrustes_ser)
export(conc_c)
export(conc_wrc)
export(cor_sq)
export(element_eval)
export(im_long)
export(im_merge)
export(im_read_csv)
export(im_ref)
export(lakhesize)
export(spearman_sq)
export(strand_add)
export(strand_create)
export(strand_extract)
export(strand_suppress)
export(strands_create)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
useDynLib(lakhesis)
