# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @useDynLib lakhesis
#' @importFrom Rcpp sourceCpp
rss_rotation <- function(x_r, ref_r, ref_mid_rad) {
    .Call('_lakhesis_rss_rotation', PACKAGE = 'lakhesis', x_r, ref_r, ref_mid_rad)
}

#' @useDynLib lakhesis
#' @importFrom Rcpp sourceCpp
orth_proj_fit <- function(x_r_rot, ref_r) {
    .Call('_lakhesis_orth_proj_fit', PACKAGE = 'lakhesis', x_r_rot, ref_r)
}

